@@ -2,62 +2,50 @@
  *
  * test dccifd via dccif()
  *
- * Copyright (c) 2005 by Rhyolite Software, LLC
+ * This is probably not the right way to use dccif(), because it uses
+ * too much internal DCC code.
  *
- * This agreement is not applicable to any entity which sells anti-spam
- * solutions to others or provides an anti-spam solution as part of a
- * security solution sold to other entities, or to a private network
- * which employs the DCC or uses data provided by operation of the DCC
- * but does not provide corresponding data to other users.
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * Parties not eligible to receive a license under this agreement can
- * obtain a commercial license to use DCC and permission to use
- * U.S. Patent 6,330,590 by contacting Commtouch at http://www.commtouch.com/
- * or by email to nospam@commtouch.com.
- *
- * A commercial license would be for Distributed Checksum and Reputation
- * Clearinghouse software.  That software includes additional features.  This
- * free license for Distributed ChecksumClearinghouse Software does not in any
- * way grant permision to use Distributed Checksum and Reputation Clearinghouse
- * software
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE, LLC DISCLAIMS ALL
- * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE, LLC
- * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
- * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
- * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
- * SOFTWARE.
- *
- * Rhyolite Software DCC 1.3.42-1.21 $Revision$
+ * --C-LICENSE--
+ * $Revision: 1.35 $
  */
 
-#include "dccif.h"
-#include "dcc_clnt.h"
-#include "dcc_paths.h"
-#include <arpa/inet.h>
 
-static void NRATTRIB
+#include "dccif.h"
+
+#if 0
+/* to test simple compiling */
+int
+main(int argc, char **argv)
+{
+	dccif(0,
+	      0, 0,
+	      0, 0, 0, 0, 0,
+	      0, 0, 0, 0);
+}
+
+#else
+
+#include "dcc_errlog.h"
+#include "sendmail-sysexits.h"
+
+
+static void DCC_NORET
 usage(void)
 {
 	dcc_logbad(EX_USAGE,
 		   "usage: [-VP] [-h FIFO | homedir | hostname,port] [-o opts]"
-		   " [-c clnt-addr]\n"
+		   " [-c clnt-name]\n"
 		   "    [-l heLo] [-f env_from] [-I ifile] [-O ofile]"
 		   " [-r rcpt1[,user1]] [...]\n");
 }
 
 
 
-int NRATTRIB
+int
 main(int argc, char **argv)
 {
 	DCC_EMSG emsg;
+	u_char print_version = 0;
 	const char *srvr_addr;
 	const char *opts;
 	DCC_SOCKU su, *sup;
@@ -78,8 +66,8 @@ main(int argc, char **argv)
 	dccproc = 0;
 	helo = 0;
 	env_from = 0;
-	in_body_fd = 0;
-	out_body_fd = dup(1);
+	in_body_fd = STDIN_FILENO;
+	out_body_fd = dup(STDOUT_FILENO);
 	rcpts = 0;
 
 	dcc_syslog_init(0, argv[0], 0);
@@ -87,7 +75,8 @@ main(int argc, char **argv)
 	while (EOF != (i = getopt(argc, argv, "VPh:o:c:l:f:I:O:r:"))) {
 		switch (i) {
 		case 'V':
-			fprintf(stderr, DCC_VERSION"\n");
+			dcc_version_print();
+			print_version = 1;
 			break;
 
 		case 'P':
@@ -107,15 +96,18 @@ main(int argc, char **argv)
 			break;
 
 		case 'c':
-			clnt_nm = optarg;
-			dcc_host_lock();
-			if (!dcc_get_host(clnt_nm, 2, &error)) {
-				dcc_logbad(EX_USAGE, "%s: %s\n", clnt_nm,
-					   DCC_HSTRERROR(error));
+			if (strcmp("0.0.0.0", optarg)) {
+				dcc_host_lock();
+				if (!dcc_get_host(optarg, DCC_GETH_DEF,
+						  &error)) {
+					dcc_logbad(EX_USAGE, "%s: %s\n",
+						   optarg, H_ERROR_STR1(error));
+				}
+				dcc_ipv4su2ipv6su(&su, &dcc_hostaddrs[0]);
+				dcc_host_unlock();
+				sup = &su;
 			}
-			dcc_ipv4sutoipv6(&su, &dcc_hostaddrs[0]);
-			sup = &su;
-			dcc_host_unlock();
+			clnt_nm = optarg;
 			break;
 
 		case 'l':
@@ -170,14 +162,20 @@ main(int argc, char **argv)
 	if (argc != 0)
 		usage();
 
-	result = dccif(emsg,
+	if (print_version)
+		exit(EX_OK);
+
+	if (!rcpts)
+		fprintf(stderr, "no recipients specified\n");
+
+	result = dccif(&emsg,
 		       out_body_fd, 0,
 		       opts, sup, clnt_nm, helo, env_from,
 		       rcpts, in_body_fd, 0, srvr_addr);
 
 	if (!dccproc) {
 		if (!result) {
-			printf("result 0: %s\n", emsg);
+			printf("result 0: %s\n", emsg.c);
 			exit(0);
 		}
 		printf("overall result = '%c'\n", result);
@@ -192,3 +190,4 @@ main(int argc, char **argv)
 	}
 	exit(0);
 }
+#endif /* 0 */
