@@ -2,42 +2,12 @@
  *
  * dccif(), a sample C interface to the DCC interface daemon, dccifd
  *
- * Copyright (c) 2005 by Rhyolite Software, LLC
- *
- * This agreement is not applicable to any entity which sells anti-spam
- * solutions to others or provides an anti-spam solution as part of a
- * security solution sold to other entities, or to a private network
- * which employs the DCC or uses data provided by operation of the DCC
- * but does not provide corresponding data to other users.
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * Parties not eligible to receive a license under this agreement can
- * obtain a commercial license to use DCC and permission to use
- * U.S. Patent 6,330,590 by contacting Commtouch at http://www.commtouch.com/
- * or by email to nospam@commtouch.com.
- *
- * A commercial license would be for Distributed Checksum and Reputation
- * Clearinghouse software.  That software includes additional features.  This
- * free license for Distributed ChecksumClearinghouse Software does not in any
- * way grant permision to use Distributed Checksum and Reputation Clearinghouse
- * software
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE, LLC DISCLAIMS ALL
- * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE, LLC
- * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
- * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
- * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
- * SOFTWARE.
- *
- * Rhyolite Software DCC 1.3.42-1.28 $Revision$
+ * --C-LICENSE--
+ * $Revision: 1.41 $
  */
 
 
+#include "dcc_defs.h"
 #include "dccif.h"
 #include "dcc_clnt.h"
 #include "sys/uio.h"
@@ -67,8 +37,8 @@ dccif_closes(int *fd1, int *fd2, int *fd3)
 
 
 
-static void PATTRIB(6,7)
-dccif_pemsg(int ex_code, DCC_EMSG emsg,
+static void DCC_PF(6,7)
+dccif_pemsg(int ex_code, DCC_EMSG *emsg,
 	    int *fd1, int *fd2, int *fd3,
 	    const char *msg, ...)
 {
@@ -86,24 +56,29 @@ dccif_pemsg(int ex_code, DCC_EMSG emsg,
 
 
 static u_char
-dccif_sock(DCC_EMSG emsg,
+dccif_sock(DCC_EMSG *emsg,
 	   const char *srvr_addr,	/* home directory, FIFO, or host,port */
-	   SOCKET *s, int* fd2, int* fd3)
+	   DCC_SOCKET *s, int* fd2, int* fd3)
 {
 	struct stat sb;
-	char host[MAXHOSTNAMELEN];
-	u_int16_t port;
+	char host[DCC_MAXDOMAINLEN];
+	in_port_t port;
 	DCC_SOCKU su;
 	struct sockaddr_un sunsock;
 	const char *cp;
 	int error;
 
-	/* assume srvr_addr is a hostname,port if not a file or directory */
+	/* srvr_addr should be a hostname,port if not a file or directory */
 	if (0 > stat(srvr_addr, &sb)) {
+		if (!strchr(srvr_addr, ',')) {
+			dccif_pemsg(EX_IOERR, emsg, s, fd2, fd3,
+				    "stat(%s): %s", srvr_addr, ERROR_STR());
+			return 0;
+		}
+
 		cp = dcc_parse_nm_port(emsg, srvr_addr, DCC_GET_PORT_INVALID,
 				       host, sizeof(host),
-				       &port, 0, 0,
-				       0, 0);
+				       &port, 0, 0, 0, 0);
 		if (!cp) {
 			dccif_closes(0, fd2, fd3);
 			return 0;
@@ -119,15 +94,14 @@ dccif_sock(DCC_EMSG emsg,
 			return 0;
 		}
 		dcc_host_lock();
-		if (!dcc_get_host(host, 2, &error)) {
+		if (!dcc_get_host(host, DCC_GETH_DEF, &error)) {
 			dcc_host_unlock();
 			dccif_pemsg(EX_NOHOST, emsg, 0, fd2, fd3,
-				    "%s: %s",
-				    srvr_addr, DCC_HSTRERROR(error));
+				    "%s: %s", srvr_addr, H_ERROR_STR1(error));
 			return 0;
 		}
 		su = dcc_hostaddrs[0];
-		*DCC_SU_PORT(&su) = port;
+		DCC_SU_PORT(&su) = port;
 		dcc_host_unlock();
 
 		*s = socket(su.sa.sa_family, SOCK_STREAM, 0);
@@ -173,7 +147,7 @@ dccif_sock(DCC_EMSG emsg,
 
 
 static u_char
-dccif_writev(DCC_EMSG emsg,
+dccif_writev(DCC_EMSG *emsg,
 	     int *out_fd, int *fd2, int *fd3,
 	     const struct iovec *iovs,
 	     int num_iovs,
@@ -196,7 +170,7 @@ dccif_writev(DCC_EMSG emsg,
 
 
 static u_char
-dccif_write(DCC_EMSG emsg,
+dccif_write(DCC_EMSG *emsg,
 	    int *out_fd, int *fd2, int *fd3,
 	    const void *buf, int buf_len)
 {
@@ -218,7 +192,7 @@ dccif_write(DCC_EMSG emsg,
 
 
 static int
-dccif_read(DCC_EMSG emsg,
+dccif_read(DCC_EMSG *emsg,
 	   int *in_fd, int *fd2, int *fd3,
 	   char *buf, int buf_len, int min_read)
 {
@@ -245,8 +219,10 @@ dccif_read(DCC_EMSG emsg,
 
 
 
+/* This function is mentioned in dccifd/dccif-test/dccif-test.c
+ *	and so cannot change lightly. */
 u_char					/* 0=failed or DCCIF_RESULT_* */
-dccif(DCC_EMSG emsg,			/* put error message here */
+dccif(DCC_EMSG *emsg,			/* put error message here */
       int out_body_fd,			/* -1 or write body to here */
       char **out_body,			/* 0 or pointer for resulting body */
       const char *opts,			/* blank separated string DCCIF_OPT_* */
@@ -277,7 +253,7 @@ dccif(DCC_EMSG emsg,			/* put error message here */
 		srvr_addr = DCC_HOMEDIR;
 
 	if (emsg)
-		*emsg = '\0';
+		emsg->c[0] = '\0';
 
 	if (!clnt_addr) {
 		clnt_str[0] = '\0';
