@@ -1,4 +1,8 @@
-/* compatibility hack for old systems that don't have hstrerror() */
+/* compatibility hack for old systems that don't have hstrerror()
+ *
+ * --C-LICENSE--
+ * $Revision: 1.6 $
+ */
 
 #include "dcc_defs.h"
 
@@ -19,7 +23,7 @@ dcc_hstrerror(int err)
     static char buf[64];
 
     if (err < 0 || err > H_NERR || h_errlist[err] == NULL) {
-	snprintf(buf, sizeof(buf), "Error %d", err);
+	snprintf(buf, sizeof(buf), "host name error %d", err);
 	return buf;
     }
     return h_errlist[err];
