@@ -17,16 +17,11 @@
  * SOFTWARE.
  */
 
-/*  Rhyolite Software DCC 1.3.42-1.5 $Revision$ */
+/*  $Revision: 1.8 $ */
 
 
 #include "dcc_defs.h"
 
-#include <sys/types.h>
-
-#ifndef DCC_WIN32
-#include <netinet/in.h>
-#endif
 #ifdef HAVE_ARPA_NAMESER_H
 #include <arpa/nameser.h>
 #endif
@@ -145,7 +140,9 @@ inet_ntop6(src, dst, size)
 	for (i = 0; i < NS_IN6ADDRSZ; i++)
 		words[i / 2] |= (src[i] << ((1 - (i % 2)) << 3));
 	best.base = -1;
+	best.len = 0;
 	cur.base = -1;
+	cur.len = 0;
 	for (i = 0; i < (NS_IN6ADDRSZ / NS_INT16SZ); i++) {
 		if (words[i] == 0) {
 			if (cur.base == -1)
