@@ -1,38 +1,7 @@
 /* Distributed Checksum Clearinghouse
  *
- * Copyright (c) 2005 by Rhyolite Software, LLC
- *
- * This agreement is not applicable to any entity which sells anti-spam
- * solutions to others or provides an anti-spam solution as part of a
- * security solution sold to other entities, or to a private network
- * which employs the DCC or uses data provided by operation of the DCC
- * but does not provide corresponding data to other users.
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * Parties not eligible to receive a license under this agreement can
- * obtain a commercial license to use DCC and permission to use
- * U.S. Patent 6,330,590 by contacting Commtouch at http://www.commtouch.com/
- * or by email to nospam@commtouch.com.
- *
- * A commercial license would be for Distributed Checksum and Reputation
- * Clearinghouse software.  That software includes additional features.  This
- * free license for Distributed ChecksumClearinghouse Software does not in any
- * way grant permision to use Distributed Checksum and Reputation Clearinghouse
- * software
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE, LLC DISCLAIMS ALL
- * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE, LLC
- * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
- * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
- * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
- * SOFTWARE.
- *
- * Rhyolite Software DCC 1.3.42-1.40 $Revision$
+ * --C-LICENSE--
+ * $Revision: 1.51 $
  */
 
 #include "dcc_defs.h"
@@ -40,13 +9,6 @@
 #include <arpa/inet.h>
 #endif
 
-#if !defined(HAVE_INET_NTOP) || defined(NO_IPV6)
-#define DCC_INET_NTOP dcc_inet_ntop
-extern const char *DCC_INET_NTOP(int, const void *, char *, size_t);
-#else
-#define DCC_INET_NTOP inet_ntop
-#endif
-
 
 
 /* strip IPv6 prefix from ::ffff:10.2.3.4 */
@@ -61,7 +23,9 @@ dcc_trim_ffff(const char *str)
 const char *
 dcc_ipv4tostr(char *buf, int buf_len, const struct in_addr *addr4)
 {
-	if (!DCC_INET_NTOP(AF_INET, addr4, buf, buf_len))
+	if (addr4->s_addr == INADDR_ANY)
+		STRLCPY(buf, DCC_INADDRANY, buf_len);
+	else if (!DCC_INET_NTOP(AF_INET, addr4, buf, buf_len))
 		STRLCPY(buf, "???", buf_len);
 	return buf;
 }
@@ -86,9 +50,7 @@ dcc_ipv6tostr2(char *buf, int buf_len, const struct in6_addr *addr6)
 	if (dcc_ipv6toipv4(&addr4, addr6))
 		return dcc_ipv4tostr(buf, buf_len, &addr4);
 
-	if (!DCC_INET_NTOP(AF_INET6, addr6, buf, buf_len))
-		STRLCPY(buf, "???", buf_len);
-	return buf;
+	return dcc_ipv6tostr(buf, buf_len, addr6);
 }
 
 
@@ -96,23 +58,22 @@ dcc_ipv6tostr2(char *buf, int buf_len, const struct in6_addr *addr6)
 const char *
 dcc_ip2str(char *buf, int buf_len, const DCC_IP *ip)
 {
+#if defined(HAVE_SIN6_SCOPE_ID) && defined(DCC_USE_GETADDRINFO)
+	DCC_SOCKU su;
+#endif
+
 	if (ip->family == AF_INET)
-		return dcc_ipv4tostr(buf, buf_len, &ip->u.v4);
-	else
-		return dcc_ipv6tostr(buf, buf_len, &ip->u.v6);
-}
+		return dcc_ipv4tostr(buf, buf_len, &ip->u.ipv4);
 
-
-
-/* convert to a string without the port number.
- *	try to make a short IPv4 string */
-const char *
-dcc_su2str2(char *buf, int buf_len, const DCC_SOCKU *su)
-{
-	if (su->sa.sa_family == AF_INET)
-		return dcc_ipv4tostr(buf, buf_len, &su->ipv4.sin_addr);
-
-	return dcc_ipv6tostr2(buf, buf_len, &su->ipv6.sin6_addr);
+#if defined(HAVE_SIN6_SCOPE_ID) && defined(DCC_USE_GETADDRINFO)
+	/* try to use getnameinfo() to deal with scope_id
+	 * ignore it if we do not have getnameinfo() */
+	dcc_ip2su(&su, ip);
+	if (!getnameinfo(&su.sa, DCC_SU_LEN(&su), buf, buf_len,
+			 0, 0, NI_NUMERICHOST))
+		return buf;
+#endif
+	return dcc_ipv6tostr(buf, buf_len, &ip->u.ipv6);
 }
 
 
@@ -127,15 +88,51 @@ dcc_su2str(char *buf, int buf_len, const DCC_SOCKU *su)
 	if (su->sa.sa_family == AF_INET) {
 		dcc_ipv4tostr(buf, buf_len, &su->ipv4.sin_addr);
 	} else {
-		dcc_ipv6tostr2(buf, buf_len, &su->ipv6.sin6_addr);
+#if defined(HAVE_SIN6_SCOPE_ID) && defined(DCC_USE_GETADDRINFO)
+		if (getnameinfo(&su->sa, DCC_SU_LEN(su), buf, buf_len,
+				0, 0, NI_NUMERICHOST))
+#endif
+			dcc_ipv6tostr2(buf, buf_len, &su->ipv6.sin6_addr);
 	}
 	i = strlen(buf);
-	snprintf(buf+i, buf_len-i, ",%d", ntohs(*DCC_SU_PORT(su)));
+	snprintf(buf+i, buf_len-i, ",%d", ntohs(DCC_SU_PORT(su)));
 	return buf;
 }
 
 
 
+/* convert to a string without the port number.
+ *	try to make a short IPv4 string */
+const char *
+dcc_su2str2(char *buf, int buf_len, const DCC_SOCKU *su)
+{
+	if (su->sa.sa_family == AF_INET)
+		return dcc_ipv4tostr(buf, buf_len, &su->ipv4.sin_addr);
+
+#if defined(HAVE_SIN6_SCOPE_ID) && defined(DCC_USE_GETADDRINFO)
+	if (!getnameinfo(&su->sa, DCC_SU_LEN(su), buf, buf_len,
+			 0, 0, NI_NUMERICHOST))
+		return buf;
+#endif
+	return dcc_ipv6tostr2(buf, buf_len, &su->ipv6.sin6_addr);
+}
+
+
+
+/* convert to a string without a boring port number
+ *	try to make a short IPv4 string */
+const char *
+dcc_su2str3(char *buf, int buf_len, const DCC_SOCKU *su, u_short port)
+{
+	if (DCC_SU_PORT(su) != port) {
+		return dcc_su2str(buf, buf_len, su);
+	} else {
+		return dcc_su2str2(buf, buf_len, su);
+	}
+}
+
+
+
 /* Convert IP address to a string, but not into a single buffer
  *	This is not thread safe but good enough for error messages */
 const char *
@@ -173,7 +170,15 @@ const char *
 dcc_su2name(char *name, int name_len, const DCC_SOCKU *su)
 {
 #undef EXPANDED
-#if defined(USE_GETIPNODEBYNAME) && !defined(EXPANDED) && !defined(NO_IPV6)
+#ifdef DCC_USE_GETADDRINFO
+#define EXPANDED
+	/* assume getnameinfo() is thread safe */
+	if (0 != getnameinfo(&su->sa, DCC_SU_LEN(su),
+			     name, name_len, 0, 0, NI_NAMEREQD)
+	    && name_len > 0)
+		*name = '\0';
+#endif
+#ifdef DCC_USE_GETIPNODEBYNAME
 #define EXPANDED
 	struct hostent *hp;
 	int error;
@@ -197,23 +202,13 @@ dcc_su2name(char *name, int name_len, const DCC_SOCKU *su)
 	}
 	dcc_host_unlock();
 #endif
-#if defined(USE_GETADDRINFO) && !defined(EXPANDED) && !defined(NO_IPV6)
+#ifdef DCC_USE_GETHOSTBYNAME
 #define EXPANDED
-	int error;
-
-	dcc_host_lock();
-	error = getnameinfo(&su->sa, DCC_SU_LEN(su),
-			    name, name_len, 0, 0, NI_NAMEREQD);
-	dcc_host_unlock();
-	if (error && name_len > 0)
-		*name = '\0';
-#endif
-#ifndef EXPANDED
 	struct hostent *hp;
 	DCC_SOCKU su4;
 
 	dcc_host_lock();
-	if (dcc_ipv6sutoipv4(&su4, su)) {
+	if (dcc_ipv6su2ipv4su(&su4, su)) {
 		hp = gethostbyaddr((char *)&su4.ipv4.sin_addr,
 				   sizeof(su4.ipv4.sin_addr),
 				   AF_INET);
@@ -224,6 +219,10 @@ dcc_su2name(char *name, int name_len, const DCC_SOCKU *su)
 		STRLCPY(name, hp ? hp->h_name : "", name_len);
 	dcc_host_unlock();
 #endif
-	return name;
+#ifndef EXPANDED
+#error no DNS service?
+#endif
 #undef EXPANDED
+
+	return name;
 }
