@@ -2,39 +2,8 @@
  *
  * routines to make WIN32 look sort of reasonable
  *
- * Copyright (c) 2005 by Rhyolite Software, LLC
- *
- * This agreement is not applicable to any entity which sells anti-spam
- * solutions to others or provides an anti-spam solution as part of a
- * security solution sold to other entities, or to a private network
- * which employs the DCC or uses data provided by operation of the DCC
- * but does not provide corresponding data to other users.
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * Parties not eligible to receive a license under this agreement can
- * obtain a commercial license to use DCC and permission to use
- * U.S. Patent 6,330,590 by contacting Commtouch at http://www.commtouch.com/
- * or by email to nospam@commtouch.com.
- *
- * A commercial license would be for Distributed Checksum and Reputation
- * Clearinghouse software.  That software includes additional features.  This
- * free license for Distributed ChecksumClearinghouse Software does not in any
- * way grant permision to use Distributed Checksum and Reputation Clearinghouse
- * software
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE, LLC DISCLAIMS ALL
- * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE, LLC
- * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
- * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
- * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
- * SOFTWARE.
- *
- * Rhyolite Software DCC 1.3.42-1.20 $Revision$
+ * --C-LICENSE--
+ * $Revision: 1.26 $
  */
 
 
@@ -49,7 +18,7 @@ static u_char is_nt;
 
 
 void
-win32_init(void)
+dcc_win32_init(void)
 {
 	OSVERSIONINFO ver;
 	WSADATA WSAData;
@@ -73,7 +42,7 @@ win32_init(void)
 
 
 int
-gettimeofday(struct timeval *tv, struct timezone *tzp)
+dcc_gettimeofday(struct timeval *tv, struct timezone *tzp)
 {
 	static SYSTEMTIME epoch_st = {1970, 1, 0, 1, 0, 0, 0, 0};
 	static LONGLONG epoch;		/* the first second of the UNIX epoch */
@@ -92,7 +61,7 @@ gettimeofday(struct timeval *tv, struct timezone *tzp)
 
 
 void
-win32_unmap(HANDLE *hp, void *p, const char *nm)
+dcc_win32_unmap(HANDLE *hp, void *p, const char *nm)
 {
 	if (!UnmapViewOfFile(p))
 		dcc_error_msg("UnmapViewOfFile(%s): %s", nm, ERROR_STR());
@@ -104,11 +73,11 @@ win32_unmap(HANDLE *hp, void *p, const char *nm)
 
 
 void *
-win32_map(DCC_EMSG emsg,
-	  HANDLE *map_handle,		/* put handle for the map here */
-	  const char *nm,		/* for this resolved path name */
-	  int fd,			/* with this C style file descriptor */
-	  int size)			/* with this size (to extend file) */
+dcc_win32_map(DCC_EMSG emsg,
+	      HANDLE *map_handle,	/* put handle for the map here */
+	      const char *nm,		/* for this resolved path name */
+	      int fd,			/* with this C style file descriptor */
+	      int size)			/* with this size (to extend file) */
 {
 	static char junk;		/* foil optimizer */
 	DCC_PATH map_nm;
@@ -117,7 +86,7 @@ win32_map(DCC_EMSG emsg,
 	int i;
 
 	/* make a name for the mapping */
-	if (!fnm2path(map_nm, nm, 0))
+	if (!dcc_fnm2abs(map_nm, nm, 0))
 		STRLCPY(map_nm, nm, sizeof(DCC_PATH));
 	for (i = 0; i < sizeof(DCC_PATH) && map_nm[i] != '\0'; ++i) {
 		if (map_nm[i] == '/' || map_nm[i] == '\\')
@@ -155,7 +124,7 @@ win32_map(DCC_EMSG emsg,
 
 /* get an exclusive lock on a file */
 u_char
-win32_lock(HANDLE h, DWORD flags)
+dcc_win32_lock(HANDLE h, DWORD flags)
 {
 	OVERLAPPED olap;
 	int e;
@@ -181,63 +150,25 @@ win32_lock(HANDLE h, DWORD flags)
 
 
 u_char
-win32_unlock(HANDLE h)
+dcc_win32_unlock(HANDLE h)
 {
 	return UnlockFile(h, 0,0, 1,0);
 }
 
 
 
-/* not all WIN32 systems have snprintf */
+/* not all WIN32 systems have snprintf
+ *	At least some versions of FormatMessage() do not understand %f
+ *	There should be no unsafe sprintf's, so worry only a little */
 int
-dcc_vsnprintf(char *buf, int buf_len, const char *pat, va_list args)
+dcc_vsnprintf(char *tgt, int tgt_len, const char *pat, va_list args)
 {
-#	define PATBUF_LEN 512
-	char c, patbuf[PATBUF_LEN+5+1], msgbuf[PATBUF_LEN*2];
-	DWORD lasterror;
-	int patlen, patnum, i;
+	char buf[32*1024];
+	int len;
 
-	/* translate from printf to FormatMessage pattern */
-	patlen = 0;
-	patnum = 0;
-	while (patnum < 20 && patlen < PATBUF_LEN && (c = *pat++) != '\0') {
-		patbuf[patlen++] = c;
-		if (c != '%')
-			continue;
-
-		if (*pat == '%') {
-			patbuf[patlen++] = '%';
-			++pat;
-			continue;
-		}
-
-		if (++patnum >= 10)
-			patbuf[patlen++] = '1';
-		patbuf[patlen++] = '0' + (patnum % 10);
-		patbuf[patlen++] = '!';
-		while (((c = *pat++) == '.' || c == '#' || c == '*'
-			|| c == '+' || c == '-' || c == 'l'
-			|| (c >= '0' && c <= '9'))
-		       && patlen < PATBUF_LEN) {
-			patbuf[patlen++] = c;
-		}
-		patbuf[patlen++] = c;
-		patbuf[patlen++] = '!';
-	}
-	patbuf[patlen] = '\0';
-
-	/* FormatMessage() fails completely if the buffer is too short
-	 * so waste time with a temporary buffer. */
-	lasterror = GetLastError();
-	i = FormatMessage(FORMAT_MESSAGE_FROM_STRING, patbuf,
-			  0, 0, msgbuf, sizeof(msgbuf), &args);
-	if (!i) {
-		sprintf(msgbuf, "FormatMessage() failed: %d", GetLastError());
-		i = strlen(msgbuf);
-	}
-	STRLCPY(buf, msgbuf, buf_len);
-	SetLastError(lasterror);
-	return i;
+	len = vsprintf(buf, pat, args);
+	STRLCPY(tgt, buf, tgt_len);
+	return len;
 }
 
 
@@ -245,26 +176,24 @@ dcc_vsnprintf(char *buf, int buf_len, const char *pat, va_list args)
 int
 dcc_snprintf(char *buf, int buf_len, const char *pat, ...)
 {
-	int i;
+	int len;
 	va_list args;
 
 	va_start(args, pat);
-	i = dcc_vsnprintf(buf, buf_len, pat, args);
+	len = dcc_vsnprintf(buf, buf_len, pat, args);
 	va_end(args);
-	return i;
+	return len;
 }
 
 
 
-
-
 /* in NT, this should probably have something to do with the event log */
 
 char syslog_prefix[64];
 
 #pragma argsused
 void
-openlog(const char *ident, int logopt, int facility)
+dcc_openlog(const char *ident, int logopt, int facility)
 {
 	BUFCPY(syslog_prefix, ident);
 }
@@ -273,7 +202,7 @@ openlog(const char *ident, int logopt, int facility)
 
 #pragma argsused
 void
-vsyslog(int priority, const char *msg, va_list args)
+dcc_vsyslog(int priority, const char *msg, va_list args)
 {
 	struct tm tm;
 	char *bp, buf[sizeof(syslog_prefix)+256];
@@ -299,19 +228,19 @@ vsyslog(int priority, const char *msg, va_list args)
 
 
 void
-syslog(int priority, const char *msg, ...)
+dcc_syslog(int priority, const char *msg, ...)
 {
 	va_list args;
 
 	va_start(args, msg);
-	vsyslog(priority, msg, args);
+	dcc_vsyslog(priority, msg, args);
 	va_end(args);
 }
 
 
 
 void
-closelog(void)
+dcc_closelog(void)
 {
 	fflush(stdout);
 	fflush(stderr);
@@ -358,7 +287,7 @@ const char *
 ws_strerror(int eno)
 {
 	static struct {
-	    char    s[150];
+	    char    s[256];
 	} ebufs[8];
 	static int ebuf_num;
 	int bn;
@@ -388,7 +317,8 @@ ws_strerror(int eno)
 		/* strip any CR or LF and convert the strange, non-ASCII
 		 * garbage from Microsoft messages */
 		ws_strip(begin, begin, begin+sizeof(ebufs[bn].s)-1);
-		return begin;
+		if (strlen(begin) < 128)
+			return begin;
 	}
 
 	/* If Microsoft fails, try the Borland messages,
@@ -443,8 +373,12 @@ ws_strerror(int eno)
 	case WSAETIMEDOUT:	return "Operation timed out";
 	case WSAETOOMANYREFS:	return "Too many references: can't splice";
 	case WSAEUSERS:		return "Too many users";
-	case WSAEWOULDBLOCK:	return "operation would block";
+	case WSAEWOULDBLOCK:	return "Operation would block";
 	case WSANOTINITIALISED:	return "WSAStartup not done";
+	case WSAHOST_NOT_FOUND:	return "Unknown host";
+	case WSATRY_AGAIN:	return "Host name lookup failure";
+	case WSANO_RECOVERY:	return "Unknown server error";
+	case WSANO_DATA:	return "No address associated with name";
 	}
 
 	/* fall back on Borland's "unkonwn error", but admit the error # */
