@@ -2,64 +2,24 @@
  *
  * threaded version of client locking
  *
- * Copyright (c) 2005 by Rhyolite Software, LLC
- *
- * This agreement is not applicable to any entity which sells anti-spam
- * solutions to others or provides an anti-spam solution as part of a
- * security solution sold to other entities, or to a private network
- * which employs the DCC or uses data provided by operation of the DCC
- * but does not provide corresponding data to other users.
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * Parties not eligible to receive a license under this agreement can
- * obtain a commercial license to use DCC and permission to use
- * U.S. Patent 6,330,590 by contacting Commtouch at http://www.commtouch.com/
- * or by email to nospam@commtouch.com.
- *
- * A commercial license would be for Distributed Checksum and Reputation
- * Clearinghouse software.  That software includes additional features.  This
- * free license for Distributed ChecksumClearinghouse Software does not in any
- * way grant permision to use Distributed Checksum and Reputation Clearinghouse
- * software
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE, LLC DISCLAIMS ALL
- * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE, LLC
- * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
- * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
- * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
- * SOFTWARE.
- *
- * Rhyolite Software DCC 1.3.42-1.59 $Revision$
+ * --C-LICENSE--
+ * $Revision: 1.83 $
  */
 
-#include "dcc_ck.h"
-#include "dcc_clnt.h"
-#ifdef HAVE_PTHREAD_H
-#include <pthread.h>
-#else
-#include <sys/pthread.h>
-#endif
-#include <signal.h>
-
-u_char grey_on;
-u_char grey_query_only;
-
-DCC_WF cmn_wf, cmn_tmp_wf;
+#include "cmn_defs.h"
 
 
 /* many POSIX thread implementations have unexpected side effects on
  * ordinary system calls, so don't use the threaded version unless
  * necessary */
 
+u_char clnt_threaded = 1;
+
+
 /* protect the links among contexts and the miscellaneous global
  * variables in the DCC client library */
 static pthread_mutex_t ctxts_mutex;
-#ifdef DCC_DEBUG_CLNT_LOCK
+#ifdef DCC_DEBUG_LOCKS
 static pthread_t ctxts_owner;
 #endif
 
@@ -75,34 +35,58 @@ static u_char malloc_threaded;
 /* make gethostbyname() thread-safe */
 static pthread_mutex_t host_mutex;
 
-/* Only one DCC client process (not just one thread in every process) can
- * resolve server host names or send a burst of NOPs to measure RTTs.
- * These locks only keep more than one thread in a process from trying to
- * resolve server names to addresses.  A lock on the map file is used
- * to resolve the lock among processes. */
-static pthread_mutex_t clnt_resolve_mutex;
-#ifdef DCC_DEBUG_CLNT_LOCK
-static pthread_t clnt_resolve_owner;
-#endif
 static pthread_t clnt_resolve_tid;
 static pthread_cond_t clnt_resolve_cond;
 static u_char clnt_resolve_stopping;
 
-/* The threaded DNS blacklist and external filter support uses fork() to
- * create helper processes to wait for the typical single-threaded DNS
- * resolver library and who knows what sort of external filter. */
-static pthread_mutex_t helper_cnt_mutex;
-static pthread_mutex_t helper_recv_mutex;
+/* The threaded DNS blacklist support uses fork() to create helper processes
+ * to wait for the typical single-threaded DNS resolver library. */
+static pthread_mutex_t helper_mutex;
+static pthread_t helper_owner;
+
+/* create user logs in a burst while holding a lock
+ *	this reduces the total number of file descriptors needed
+ *	at a cost of stopping everything while copying from the main
+ *	log file to the per-user log files */
+pthread_mutex_t user_log_mutex;
+pthread_t user_log_owner;
+
+
+/* this is used only in the threaded DCC clients */
+void
+clnt_sigs_off(sigset_t *sigsold)
+{
+	sigset_t sigsnew;
+	int error;
+
+	sigemptyset(&sigsnew);
+	sigaddset(&sigsnew, SIGHUP);
+	sigaddset(&sigsnew, SIGINT);
+	sigaddset(&sigsnew, SIGTERM);
+	sigaddset(&sigsnew, SIGALRM);
+	error = pthread_sigmask(SIG_BLOCK, &sigsnew, sigsold);
+	if (error)
+		dcc_logbad(EX_SOFTWARE, "pthread_sigmask(): %s",
+			   ERROR_STR1(error));
+}
+
 
 
 void
 dcc_ctxts_lock(void)
 {
-	int error = pthread_mutex_lock(&ctxts_mutex);
+	int error;
+
+#ifdef DCC_DEBUG_LOCKS
+	if (ctxts_owner == pthread_self())
+		dcc_logbad(EX_SOFTWARE, "already have ctxts lock");
+#endif
+
+	error = pthread_mutex_lock(&ctxts_mutex);
 	if (error)
 		dcc_logbad(EX_SOFTWARE, "pthread_mutex_lock(ctxts): %s",
 			   ERROR_STR1(error));
-#ifdef DCC_DEBUG_CLNT_LOCK
+#ifdef DCC_DEBUG_LOCKS
 	ctxts_owner = pthread_self();
 #endif
 }
@@ -114,7 +98,7 @@ dcc_ctxts_unlock(void)
 {
 	int error;
 
-#ifdef DCC_DEBUG_CLNT_LOCK
+#ifdef DCC_DEBUG_LOCKS
 	ctxts_owner = 0;
 #endif
 	error = pthread_mutex_unlock(&ctxts_mutex);
@@ -125,13 +109,22 @@ dcc_ctxts_unlock(void)
 
 
 
-#ifdef DCC_DEBUG_CLNT_LOCK
+#ifdef DCC_DEBUG_LOCKS
 void
-have_ctxts_lock(void)
+assert_ctxts_locked(void)
 {
 	if (ctxts_owner != pthread_self())
 		dcc_logbad(EX_SOFTWARE, "don't have ctxts lock");
 }
+
+
+
+void
+assert_ctxts_unlocked(void)
+{
+	if (ctxts_owner == pthread_self())
+		dcc_logbad(EX_SOFTWARE, "have ctxts lock");
+}
 #endif
 
 
@@ -141,7 +134,7 @@ dcc_syslog_lock(void)
 {
 	int error;
 
-	if (!syslog_threaded)
+	if (!clnt_threaded || !syslog_threaded)
 		return;
 	error = pthread_mutex_lock(&syslog_mutex);
 	if (error)
@@ -156,7 +149,7 @@ dcc_syslog_unlock(void)
 {
 	int error;
 
-	if (!syslog_threaded)
+	if (!clnt_threaded || !syslog_threaded)
 		return;
 	error = pthread_mutex_unlock(&syslog_mutex);
 	if (error)
@@ -173,6 +166,8 @@ u_char dcc_host_locked = 1;
  * been initialized */
 static u_char dcc_host_threaded = 0;
 
+/* This function is mentioned in dccifd/dccif-test/dccif-test.c
+ *	and so cannot change lightly. */
 void
 dcc_host_lock(void)
 {
@@ -190,6 +185,8 @@ dcc_host_lock(void)
 
 
 
+/* This function is mentioned in dccifd/dccif-test/dccif-test.c
+ *	and so cannot change lightly. */
 void
 dcc_host_unlock(void)
 {
@@ -213,7 +210,7 @@ dcc_malloc_lock(void)
 {
 	int error;
 
-	if (!malloc_threaded)
+	if (!malloc_threaded)		/* no locking until locks created */
 		return;
 	error = pthread_mutex_lock(&malloc_mutex);
 	if (error)
@@ -227,7 +224,7 @@ dcc_malloc_unlock(void)
 {
 	int error;
 
-	if (!malloc_threaded)
+	if (!malloc_threaded)		/* no locking until locks created */
 		return;
 	error = pthread_mutex_unlock(&malloc_mutex);
 	if (error)
@@ -240,7 +237,7 @@ dcc_malloc_unlock(void)
 
 #ifndef HAVE_LOCALTIME_R
 /* make localtime() thread safe */
-static pthread_mutex_t syslog_mutex;
+static pthread_mutex_t localtime_mutex;
 static u_char localtime_threaded;
 
 void
@@ -274,123 +271,148 @@ dcc_localtime_unlock(void)
 
 
 
-void
-dcc_clnt_resolve_mutex_lock(void)
-{
-	int error;
-
-	error = pthread_mutex_lock(&clnt_resolve_mutex);
-	if (error)
-		dcc_logbad(EX_SOFTWARE,
-			   "pthread_mutex_lock(resolve): %s",
-			   ERROR_STR1(error));
-#ifdef DCC_DEBUG_CLNT_LOCK
-	clnt_resolve_owner = pthread_self();
-#endif
-}
-
-
-
-void
-dcc_clnt_resolve_mutex_unlock(void)
-{
-	int error;
-
-#ifdef DCC_DEBUG_CLNT_LOCK
-	clnt_resolve_owner = 0;
-#endif
-	error = pthread_mutex_unlock(&clnt_resolve_mutex);
-	if (error)
-		dcc_logbad(EX_SOFTWARE,
-			   "pthread_mutex_unlock(resolve): %s",
-			   ERROR_STR1(error));
-}
-
-
-
-#ifdef DCC_DEBUG_CLNT_LOCK
-void
-have_clnt_resolve_mutex(void)
-{
-	if (clnt_resolve_owner != pthread_self())
-		dcc_logbad(EX_SOFTWARE, "don't have resolve mutex");
-}
-#endif
-
-
-
 const char *main_white_nm;
+const char *mapfile_nm = DCC_MAP_NM_DEF;
 
-static void * NRATTRIB
-clnt_resolve_thread(void *arg UATTRIB)
+/* resolve things */
+static DCC_CLNT_CTXT *
+resolve_sub(DCC_CLNT_CTXT *ctxt,	/* 0=allocate and initialize */
+	    WF *wf, WF *tmp_wf)
 {
-	DCC_WF wf, tmp_wf;
 	DCC_EMSG emsg;
+
+	if (!ctxt) {
+		wf_init(wf, 0);
+		if (main_white_nm)
+			dcc_new_white_nm(&emsg, wf, main_white_nm);
+
+		emsg.c[0] = '\0';
+		if (!dcc_map_info(&emsg, mapfile_nm, -1))
+			dcc_logbad(EX_USAGE, "%s", emsg.c);
+
+		ctxt = dcc_alloc_ctxt();
+	}
+
+	if (wf->ascii_nm.c[0] != '\0') {
+		if (clnt_resolve_stopping)
+			return ctxt;
+		dcc_ctxts_unlock();
+		switch (wf_rdy(&emsg, wf, tmp_wf)) {
+		case WHITE_OK:
+		case WHITE_NOFILE:
+		case WHITE_SILENT:
+			break;
+		case WHITE_CONTINUE:
+		case WHITE_COMPLAIN:
+			dcc_error_msg("%s", emsg.c);
+			break;
+		}
+		dcc_ctxts_lock();
+
+		/* Tell the other threads that the whitelist hash table
+		 * in the disk file has changed.
+		 * This kludge lets this thread use its own
+		 * wf structure without hogging the lock on cmn_wf. */
+		if (wf->closed) {
+			wf->closed = 0;
+			cmn_wf.need_reopen = 1;
+		}
+	}
+
+	return ctxt;
+}
+
+
+
+static void * DCC_NORET
+clnt_resolve_thread(void *arg DCC_UNUSED)
+{
+	WF wf, tmp_wf;
 	DCC_CLNT_CTXT *ctxt;
-	sigset_t sigsoff;
+#ifndef DCC_WIN32
+	struct stat sb;
+#endif
+	DCC_EMSG emsg;
 	int error;
 
 	/* let the thread in charge of signals deal with them */
-	sigemptyset(&sigsoff);
-	sigaddset(&sigsoff, SIGHUP);
-	sigaddset(&sigsoff, SIGINT);
-	sigaddset(&sigsoff, SIGTERM);
-	error = pthread_sigmask(SIG_BLOCK, &sigsoff, 0);
-	if (error)
-		dcc_logbad(EX_SOFTWARE, "pthread_sigmask(resolve): %s",
-			   ERROR_STR1(error));
-
-	dcc_wf_init(&wf, 0);
-	if (main_white_nm)
-		dcc_new_white_nm(emsg, &wf, main_white_nm);
+	clnt_sigs_off(0);
 
+	ctxt = 0;
 	dcc_ctxts_lock();
-	ctxt = dcc_alloc_ctxt();
-	dcc_ctxts_unlock();
 	for (;;) {
-		dcc_ctxts_lock();
+		if (clnt_resolve_stopping) {
+			if (ctxt) {
+				dcc_rel_ctxt(ctxt);
+				ctxt = 0;
+			}
+			dcc_ctxts_unlock();
+			pthread_exit(0);
+		}
+
+#ifndef DCC_WIN32
+		/* Notice if the file has been unlinked and try to reopen it.
+		 * Open files cannot be unlinked in WIN32, which lets us not
+		 * worry about whether WIN32 files have device and i-numbers */
+		if (ctxt && !(dcc_clnt_info->fgs & DCC_INFO_FG_TMP)) {
+			if (!dcc_ck_private(0, &sb, dcc_info_nm.c, -1)
+			    || sb.st_dev != dcc_info_sb.st_dev
+			    || sb.st_ino != dcc_info_sb.st_ino) {
+				dcc_error_msg("%s disappeared",
+					      dcc_info_nm.c);
+				if (ctxt) {
+					dcc_rel_ctxt(ctxt);
+					ctxt = 0;
+				}
+				dcc_unmap_close_info(0);
+			}
+		}
+#endif
+
+		ctxt = resolve_sub(ctxt, &wf, &tmp_wf);
+
+		if (clnt_resolve_stopping) {
+			if (ctxt) {
+				dcc_rel_ctxt(ctxt);
+				ctxt = 0;
+			}
+			dcc_ctxts_unlock();
+			pthread_exit(0);
+		}
+		emsg.c[0] = '\0';
+		if (!dcc_clnt_rdy(&emsg, ctxt, DCC_CLNT_FG_NO_FAIL))
+			dcc_error_msg("%s", emsg.c);
+		else if (!dcc_info_unlock(&emsg))
+			dcc_logbad(emsg_ex_code(&emsg), "%s", emsg.c);
+
+		if (grey_on) {
+			if (clnt_resolve_stopping) {
+				if (ctxt) {
+					dcc_rel_ctxt(ctxt);
+					ctxt = 0;
+				}
+				dcc_ctxts_unlock();
+				pthread_exit(0);
+			}
+			emsg.c[0] = '\0';
+			if (!dcc_clnt_rdy(&emsg, ctxt, (DCC_CLNT_FG_GREY
+						       | DCC_CLNT_FG_NO_FAIL)))
+				dcc_error_msg("%s", emsg.c);
+			else if (!dcc_info_unlock(&emsg))
+				dcc_logbad(emsg_ex_code(&emsg), "%s", emsg.c);
+		}
+
+#ifdef DCC_DEBUG_LOCKS
+		ctxts_owner = 0;
+#endif
 		error = pthread_cond_wait(&clnt_resolve_cond, &ctxts_mutex);
 		if (error != 0)
 			dcc_logbad(EX_SOFTWARE,
 				   "pthread_cond_wait(resolve): %s",
 				   ERROR_STR1(error));
-#ifdef DCC_DEBUG_CLNT_LOCK
+#ifdef DCC_DEBUG_LOCKS
 		ctxts_owner = pthread_self();
 #endif
-
-		if (clnt_resolve_stopping) {
-			dcc_ctxts_unlock();
-			pthread_exit(0);
-		}
-
-		if (!dcc_clnt_rdy(emsg, ctxt, DCC_CLNT_FG_NO_FAIL))
-			dcc_error_msg("%s", emsg);
-		else if (!dcc_info_unlock(emsg))
-			dcc_logbad(dcc_ex_code, "%s", emsg);
-
-		if (grey_on) {
-			if (!dcc_clnt_rdy(emsg, ctxt, (DCC_CLNT_FG_GREY
-						       | DCC_CLNT_FG_NO_FAIL)))
-				dcc_error_msg("%s", emsg);
-			else if (!dcc_info_unlock(emsg))
-				dcc_logbad(dcc_ex_code, "%s", emsg);
-		}
-
-		dcc_ctxts_unlock();
-
-		if (wf.ascii_nm[0] != '\0') {
-			switch (dcc_rdy_white(emsg, &wf, &tmp_wf)) {
-			case DCC_WHITE_OK:
-			case DCC_WHITE_NOFILE:
-			case DCC_WHITE_SILENT:
-				break;
-			case DCC_WHITE_CONTINUE:
-			case DCC_WHITE_COMPLAIN:
-				dcc_error_msg("%s", emsg);
-				break;
-			}
-			cmn_wf.changed = 1;
-		}
 	}
 }
 
@@ -401,8 +423,9 @@ dcc_clnt_wake_resolve(void)
 {
 	int error;
 
-	/* we cannot awaken ourself */
-	if (pthread_equal(pthread_self(), clnt_resolve_tid))
+	/* we cannot awaken ourself or awaken the thread before it starts */
+	if (clnt_resolve_tid == 0
+	    || pthread_equal(pthread_self(), clnt_resolve_tid))
 		return 0;
 
 	error = pthread_cond_signal(&clnt_resolve_cond);
@@ -417,8 +440,9 @@ dcc_clnt_wake_resolve(void)
 void
 dcc_clnt_stop_resolve(void)
 {
-	if (clnt_resolve_stopping++)
+	if (clnt_resolve_stopping)
 		return;
+	clnt_resolve_stopping = 1;
 	if (pthread_equal(pthread_self(), clnt_resolve_tid))
 		return;
 	pthread_cond_signal(&clnt_resolve_cond);
@@ -440,9 +464,7 @@ dcc_mutex_init(void *mutex, const char *nm)
 
 
 static pthread_mutex_t work_mutex;
-#ifdef DCC_DEBUG_CLNT_LOCK
 static pthread_t cwf_owner;
-#endif
 
 void
 lock_work(void)
@@ -476,9 +498,7 @@ lock_wf(void)
 	if (error)
 		dcc_logbad(EX_SOFTWARE, "pthread_mutex_lock(cwf): %s",
 			   ERROR_STR1(error));
-#ifdef DCC_DEBUG_CLNT_LOCK
 	cwf_owner = pthread_self();
-#endif
 }
 
 
@@ -487,9 +507,7 @@ void
 unlock_wf(void)
 {
 	int error;
-#ifdef DCC_DEBUG_CLNT_LOCK
 	cwf_owner = 0;
-#endif
 	error = pthread_mutex_unlock(&cwf_mutex);
 	if (error)
 		dcc_logbad(EX_SOFTWARE, "pthread_mutex_unlock(cwf): %s",
@@ -498,9 +516,9 @@ unlock_wf(void)
 
 
 
-#ifdef DCC_DEBUG_CLNT_LOCK
+#ifdef DCC_DEBUG_LOCKS
 void
-have_cwf_lock(void)
+assert_cwf_locked(void)
 {
 	if (cwf_owner != pthread_self())
 		dcc_logbad(EX_SOFTWARE, "don't have cwf lock");
@@ -512,34 +530,42 @@ have_cwf_lock(void)
 void
 dcc_clnt_thread_init(void)
 {
+	DCC_CLNT_CTXT *ctxt;
 	int error;
 
-#ifdef DCC_WIN32
-	win32_init();
-#endif
-
-	dcc_mutex_init(&ctxts_mutex, 0);
-	dcc_mutex_init(&syslog_mutex, 0);
-	syslog_threaded = 1;
-#ifdef DCC_DEBUG_HEAP
-	dcc_mutex_init(&malloc_mutex, 0);
-	malloc_threaded = 1;
-#endif
-#ifndef HAVE_LOCALTIME_R
-	dcc_mutex_init(&localtime_mutex, 0);
-	localtime_threaded = 1;
-#endif
-	dcc_mutex_init(&host_mutex, 0);
-	dcc_host_threaded = 1;
-	dcc_host_locked = 0;
-
 	/* Some pthreads implementations (e.g. AIX) don't like static
 	 * POSIX thread initializations */
 
-	dcc_mutex_init(&work_mutex, "wf_mutex");
+	dcc_mutex_init(&ctxts_mutex, "ctxt");
+	dcc_mutex_init(&syslog_mutex, "syslog");
+	syslog_threaded = 1;
+#ifdef DCC_DEBUG_HEAP
+	dcc_mutex_init(&malloc_mutex, "heap");
+	malloc_threaded = 1;
+#endif
+#ifndef HAVE_LOCALTIME_R
+	dcc_mutex_init(&localtime_mutex, "localtime");
+	localtime_threaded = 1;
+#endif
+	dcc_mutex_init(&host_mutex, "host");
+	dcc_host_threaded = 1;
+	dcc_host_locked = 0;
+
+	dcc_mutex_init(&user_log_mutex, "user_log");
+
+	dcc_mutex_init(&work_mutex, "work_mutex");
 	dcc_mutex_init(&cwf_mutex, "cwf_mutex");
 
-	dcc_mutex_init(&clnt_resolve_mutex, 0);
+	/* prevent race between resolver thread and other threads to
+	 * initialize things by doing it before starting the thread */
+	lock_wf();
+	dcc_ctxts_lock();
+	ctxt = resolve_sub(0, &cmn_wf, &cmn_tmp_wf);
+	dcc_rel_ctxt(ctxt);
+	ctxt = 0;
+	dcc_ctxts_unlock();
+	unlock_wf();
+
 	error = pthread_cond_init(&clnt_resolve_cond, 0);
 	if (error)
 		dcc_logbad(EX_SOFTWARE, "phtread_cond_init(resolve): %s",
@@ -558,73 +584,58 @@ dcc_clnt_thread_init(void)
 
 /* protect DNS blacklist helper channels */
 u_char
-helper_cnt_lock_init(void)
+helper_lock_init(void)
 {
-	dcc_mutex_init(&helper_cnt_mutex, 0);
+	dcc_mutex_init(&helper_mutex, "helper");
 	return 1;
 }
 
 
 
 void
-helper_cnt_lock(void)
+helper_lock(void)
 {
 	int error;
 
-	error =  pthread_mutex_lock(&helper_cnt_mutex);
+	assert_helper_unlocked();
+	error =  pthread_mutex_lock(&helper_mutex);
 	if (error)
-		dcc_logbad(EX_SOFTWARE,
-			   "pthread_mutex_lock(helper counter): %s",
+		dcc_logbad(EX_SOFTWARE, "pthread_mutex_lock(helper): %s",
 			   ERROR_STR1(error));
+	helper_owner = pthread_self();
 }
 
 
 
 void
-helper_cnt_unlock(void)
+helper_unlock(void)
 {
 	int error;
 
-	error = pthread_mutex_unlock(&helper_cnt_mutex);
+	assert_helper_locked();
+	helper_owner = 0;
+	error = pthread_mutex_unlock(&helper_mutex);
 	if (error)
-		dcc_logbad(EX_SOFTWARE,
-			   "pthread_mutex_unlock(helper counter): %s",
+		dcc_logbad(EX_SOFTWARE, "pthread_mutex_unlock(helper): %s",
 			   ERROR_STR1(error));
 }
 
 
 
-u_char
-helper_recv_lock_init(void)
+#ifdef DCC_DEBUG_LOCKS
+void
+assert_helper_locked(void)
 {
-	dcc_mutex_init(&helper_recv_mutex, 0);
-	return 1;
+	if (helper_owner != pthread_self())
+		dcc_logbad(EX_SOFTWARE, "don't have helper lock");
 }
 
 
 
 void
-helper_recv_lock(void)
+assert_helper_unlocked(void)
 {
-	int error;
-
-	error =  pthread_mutex_lock(&helper_recv_mutex);
-	if (error)
-		dcc_logbad(EX_SOFTWARE,
-			   "pthread_mutex_lock(helper recv): %s",
-			   ERROR_STR1(error));
-}
-
-
-
-void
-helper_recv_unlock(void)
-{
-	int error;
-
-	error = pthread_mutex_unlock(&helper_recv_mutex);
-	if (error)
-		dcc_logbad(EX_SOFTWARE,
-			   "pthread_mutex_unlock(helper recv): %s",
-			   ERROR_STR1(error));
+	if (helper_owner == pthread_self())
+		dcc_logbad(EX_SOFTWARE, "have helper lock");
 }
+#endif
