@@ -2,43 +2,13 @@
  *
  * count work
  *
- * Copyright (c) 2005 by Rhyolite Software, LLC
- *
- * This agreement is not applicable to any entity which sells anti-spam
- * solutions to others or provides an anti-spam solution as part of a
- * security solution sold to other entities, or to a private network
- * which employs the DCC or uses data provided by operation of the DCC
- * but does not provide corresponding data to other users.
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * Parties not eligible to receive a license under this agreement can
- * obtain a commercial license to use DCC and permission to use
- * U.S. Patent 6,330,590 by contacting Commtouch at http://www.commtouch.com/
- * or by email to nospam@commtouch.com.
- *
- * A commercial license would be for Distributed Checksum and Reputation
- * Clearinghouse software.  That software includes additional features.  This
- * free license for Distributed ChecksumClearinghouse Software does not in any
- * way grant permision to use Distributed Checksum and Reputation Clearinghouse
- * software
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE, LLC DISCLAIMS ALL
- * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE, LLC
- * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
- * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
- * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
- * SOFTWARE.
- *
- * Rhyolite Software DCC 1.3.42-1.13 $Revision$
+ * --C-LICENSE--
+ * $Revision: 1.22 $
  */
 
 
 #include "cmn_defs.h"
+#include "helper.h"
 #include <signal.h>
 
 
@@ -47,20 +17,21 @@ static time_t signaled;
 static pthread_t totals_tid;
 static u_char stopping;
 
+static void totals_msg(void);
 static void totals_msg_signal(int);
 static void *totals_msg_thread(void *);
 
 
 
 static void
-totals_msg_signal(int sig UATTRIB)
+totals_msg_signal(int sig DCC_UNUSED)
 {
 	time_t now;
 
 	signal(SIGUSR1, totals_msg_signal);
 
-	now = time(0);
 	/* ignore signals faster than 60 seconds */
+	now = time(0);
 	if (!signaled || now > signaled+60) {
 		signaled = now;
 		totals.msg_next = signaled;
@@ -69,26 +40,24 @@ totals_msg_signal(int sig UATTRIB)
 
 
 
-static void * NRATTRIB
-totals_msg_thread(void *ign UATTRIB)
+static void * DCC_NORET
+totals_msg_thread(void *ign DCC_UNUSED)
 {
-	time_t now, next;
+	time_t now, next, last_clean;
 	struct tm tm, signaled_tm;
-	sigset_t sigson;
-	int error, secs;
+	int secs;
 
-	sigemptyset(&sigson);
-	sigaddset(&sigson, SIGHUP);
-	sigaddset(&sigson, SIGINT);
-	sigaddset(&sigson, SIGTERM);
-	error = pthread_sigmask(SIG_BLOCK, &sigson, 0);
-	if (error)
-		dcc_logbad(EX_SOFTWARE, "pthread_sigmask(resolve): %s",
-			   ERROR_STR1(error));
+	clnt_sigs_off(0);
 
+	last_clean = time(0);
 	for (;;) {
 		if (stopping)
 			pthread_exit(0);
+
+		helper_lock();
+		reap_helpers();
+		helper_unlock();
+
 		now = time(0);
 		next = totals.msg_next;
 		if (now >= next && next != 0) {
@@ -119,12 +88,20 @@ totals_msg_thread(void *ign UATTRIB)
 
 		}
 
-		/* reap helpers frequently */
+		/* tell dccifd or dccm to close old sockets to recover
+		 * from dictionary attacks */
+		if (last_clean > now || last_clean+5*60 < now) {
+			last_clean = now;
+			work_clean();
+		}
+
 		secs = next - now;
-		if (secs <= 0 || secs > 5)
-			secs = 5;
-		sleep(secs);
-		reap_helpers(0);
+		if (secs > 0) {
+			/* reap helpers regularly */
+			if (secs > HELPER_AUTO_REAP)
+				secs = HELPER_AUTO_REAP;
+			sleep(secs);
+		}
 	}
 }
 
@@ -154,36 +131,31 @@ void
 totals_stop(void)
 {
 	stopping = 1;
+
+	helper_lock();
+	stop_helpers();
+	helper_unlock();
+
 	totals_msg();
 }
 
 
 
 /* brag about our accomplishments */
-void
+static void
 totals_msg(void)
 {
 	time_t now;
-	struct tm tm;
 	char tbuf[20];
 	char gbuf[80];
-	sigset_t sigsoff, sigsold;
+	sigset_t sigsold;
 	int error;
 
-	sigemptyset(&sigsoff);
-	sigaddset(&sigsoff, SIGHUP);
-	sigaddset(&sigsoff, SIGINT);
-	sigaddset(&sigsoff, SIGTERM);
-	error = pthread_sigmask(SIG_BLOCK, &sigsoff, &sigsold);
-	if (error)
-		dcc_logbad(EX_SOFTWARE, "pthread_sigmask(totals): %s",
-			   ERROR_STR1(error));
+	clnt_sigs_off(&sigsold);
 
 	lock_work();
 	now = time(0);
 
-	strftime(tbuf, sizeof(tbuf), "%x %X",
-		 dcc_localtime(totals.msg_prev, &tm));
 	if (grey_on)
 		snprintf(gbuf, sizeof(gbuf),
 			 " greylist embargoed %d messages to %d targets;",
@@ -198,7 +170,9 @@ totals_msg(void)
 		      gbuf,
 		      totals.msgs_spam, totals.tgts_ignored,
 		      totals.tgts_rejected, totals.tgts_discarded,
-		      totals.msgs, totals.tgts, tbuf);
+		      totals.msgs, totals.tgts,
+		      dcc_time2str(tbuf, sizeof(tbuf), "%x %X",
+				   totals.msg_prev));
 
 	memset(&totals, 0, sizeof(totals));
 	totals.msg_prev = now;
